setwd ('/home/allgoodguys/Documents/Studying/Lund_PhD/conferences-and-events/Bayes@Lund_2018')
library(lme4)
library(brms)

df_master = read.csv('real-fake_indiv-answers.csv')
df = droplevels(df_master[df_master$noisy == T, ])  # with masking noise only
df = df[, c('id', 'sound', 'corpus', 'real')] # simplify the dataset, including only the variables relevant to this model
df$corpus = factor(df$corpus, levels = c('ut', 'hawk', 'belin', 'cordaro', 'lima', 'maurage', 'simon'))
head(df)
summary(df)

# RESEARCH QUESTION: which corpora are >likely to be rated as real?
## Descriptives
aggregate(real ~ corpus, df, mean)
obs = aggregate(real ~ corpus + sound, df, mean) 
obs$real = obs$real * 100  # convert to %
plot(obs$corpus, obs$real)

plot(aggregate(real ~ corpus, df, mean))

## Non-Bayesian GLMM
mod0 = lme4::glmer(real ~ corpus + (1|sound) + (1|id), df, family = 'binomial')
summary(mod0)
drop1(mod0, test='Chisq')

## Bayesian
mod = brms::brm(
  real ~ corpus + (1|sound) + (1|id), 
  data = df, family = 'bernoulli', 
  prior = set_prior('normal(0, 3)'),
  iter = 1000, chains = 4, cores = 4
)   # running time: 50 s + compilation
# to avoid waiting the next time, save the model:
# saveRDS(mod, 'mod_real~corpus_noise.RDS')  
# mod = readRDS('mod_real~corpus_noise.RDS')


## Checking convergence and fit
summary(mod)
plot(mod)
pp = brms::pp_check(mod)
pp + theme_bw()
# stancode(mod)


## Plotting model predictions
brms::marginal_effects(mod)
# get fitted values
newdata = data.frame(corpus = levels(df$corpus))
fit = fitted(mod, newdata = newdata, re_formula = NA) * 100  # convert to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
df_plot = cbind(newdata, fit)
df_plot

ggplot(df_plot, aes(x = corpus, y = fit)) +
  geom_violin(data=obs, aes(x=corpus, y=real), alpha=0.5, color="gray70", fill='gray95') +
  geom_jitter(data=obs, aes(x=corpus, y=real), alpha=0.3, position = position_jitter(width = 0.07)) +
  geom_errorbar(aes(ymin=lwr, ymax=upr), position=position_dodge(), size=1, width=.5) +
  geom_point(shape=21, size=4, fill='red') +
  geom_hline(yintercept=50, linetype=2) +
  xlab("") +
  ylab('Probability of responding REAL, %') +
  ylim(0,100) +
  theme_bw () +
  theme(panel.grid = element_blank())


## Contrasts between corpora
fit1 = as.data.frame(fitted(mod, newdata = newdata, re_formula = NA, summary = FALSE))
colnames(fit1) = newdata$corpus
head(fit1)
nrow(fit1)

ut_vs_rest = fit1$ut - (fit1$belin + fit1$cordaro + fit1$hawk + fit1$lima + fit1$maurage + fit1$simon) / 6
hist(ut_vs_rest)
quantile(ut_vs_rest, probs = c(.5, .025, .975)) # 33.3% [28.5, 38.3]

hawk_vs_rest = fit1$hawk - (fit1$belin + fit1$cordaro + fit1$lima + fit1$maurage + fit1$simon) / 5
quantile(hawk_vs_rest, probs = c(.5, .025, .975)) # 9.1% [-1.5, 21.4]
mean(hawk_vs_rest > 0)

ut_vs_hawk = fit1$ut - fit1$hawk
quantile(ut_vs_hawk, probs = c(.5, .025, .975)) # 25.8% [14.2, 36.5]



## Doing the same thing manually
coda = brms::posterior_samples(mod)
# colnames(coda)
coda = coda[, 1:7] # focus only on population-level (main) effects
head(coda)

antilogit = function(x) 1 / (1 + exp(-x))
fit2 = data.frame(
  ut =      antilogit(coda[, 1]),
  hawk =    antilogit(coda[, 1] + coda[, 2]),
  belin =   antilogit(coda[, 1] + coda[, 3]),
  cordaro = antilogit(coda[, 1] + coda[, 4]),
  lima =    antilogit(coda[, 1] + coda[, 5]),
  maurage = antilogit(coda[, 1] + coda[, 6]),
  simon =   antilogit(coda[, 1] + coda[, 7])
)
head(fit2)
# ...and repeat the same as with fit1
